import unittest
from datetime import date

from vha_toolbox import (
    get_last_day_of_year,
    get_first_day_of_year,
    get_first_day_of_month,
    get_last_day_of_month,
    get_last_day_of_quarter,
    get_first_day_of_quarter,
    is_renewal_due
)


class DateUtilsTestCase(unittest.TestCase):
    def test_get_first_day_of_year(self):
        dt = date(2023, 6, 15)
        result = get_first_day_of_year(dt)
        self.assertEqual(result, date(2023, 1, 1))

    def test_get_last_day_of_year(self):
        dt = date(2023, 6, 15)
        result = get_last_day_of_year(dt)
        self.assertEqual(result, date(2023, 12, 31))

    def test_get_first_day_of_quarter(self):
        dt = date(2023, 6, 15)
        result = get_first_day_of_quarter(dt)
        self.assertEqual(result, date(2023, 4, 1))

    def test_get_last_day_of_quarter(self):
        dt = date(2023, 6, 15)
        result = get_last_day_of_quarter(dt)
        self.assertEqual(result, date(2023, 6, 30))

    def test_get_first_day_of_month(self):
        dt = date(2023, 6, 15)
        result = get_first_day_of_month(dt)
        self.assertEqual(result, date(2023, 6, 1))

    def test_get_first_day_of_month_2(self):
        dt = date(2024, 2, 10)
        result = get_first_day_of_month(dt)
        self.assertEqual(result, date(2024, 2, 1))

    def test_get_last_day_of_month(self):
        dt = date(2023, 6, 15)
        result = get_last_day_of_month(dt)
        self.assertEqual(result, date(2023, 6, 30))

    def test_get_last_day_of_month_2(self):
        dt = date(2024, 2, 10)
        result = get_last_day_of_month(dt)
        self.assertEqual(result, date(2024, 2, 29))

    def test_get_last_day_of_month_3(self):
        dt = date(2021, 2, 10)
        result = get_last_day_of_month(dt)
        self.assertEqual(result, date(2021, 2, 28))

    # Existing tests
    def test_is_renewal_due_same_day(self):
        self.assertFalse(is_renewal_due(date(2023, 6, 15), date(2023, 6, 15)))

    def test_is_renewal_due_next_day(self):
        self.assertFalse(is_renewal_due(date(2023, 6, 15), date(2023, 6, 16)))

    def test_is_renewal_due_next_month(self):
        self.assertTrue(is_renewal_due(date(2023, 6, 15), date(2023, 7, 15)))

    def test_is_renewal_due_next_year(self):
        self.assertTrue(is_renewal_due(date(2023, 6, 15), date(2024, 7, 15)))

    def test_leap_year_feb_29(self):
        self.assertTrue(is_renewal_due(date(2020, 2, 29), date(2021, 2, 28)))
        self.assertFalse(is_renewal_due(date(2020, 2, 29), date(2021, 3, 1)))

    def test_non_leap_year_feb_28(self):
        self.assertFalse(is_renewal_due(date(2023, 2, 28), date(2024, 2, 27)))
        self.assertTrue(is_renewal_due(date(2023, 2, 28), date(2024, 2, 28)))

    def test_30_day_month_to_31_day_month(self):
        self.assertTrue(is_renewal_due(date(2023, 4, 30), date(2024, 4, 30)))
        self.assertFalse(is_renewal_due(date(2023, 4, 30), date(2024, 5, 1)))

    def test_31_day_month_to_30_day_month(self):
        self.assertFalse(is_renewal_due(date(2023, 5, 31), date(2024, 5, 30)))
        self.assertTrue(is_renewal_due(date(2023, 5, 31), date(2024, 5, 31)))

    def test_past_date(self):
        self.assertTrue(is_renewal_due(date(2020, 1, 1), date(2024, 1, 1)))

    def test_future_date(self):
        self.assertFalse(is_renewal_due(date(2025, 1, 1), date(2024, 1, 1)))



if __name__ == '__main__':
    unittest.main()



import math
from typing import Dict
import re


def truncate_with_ellipsis(
        input_string: str,
        max_length: int,
        ellipsis: str = '...',
        del_blank: bool = True,
) -> str:
    """
    Truncates a string with an ellipsis if it exceeds the maximum length.

    Args:
        input_string (str): The string to truncate.
        max_length (int): The maximum length of the string.
        ellipsis (str, optional): The ellipsis to use. Defaults to '...'.
        del_blank (bool, optional): Whether to delete trailing whitespace. Defaults to True.

    Returns:
        str: The truncated string.

    Examples:
        >>> truncate_with_ellipsis('Hello world!', 5)
        'Hello...'
        >>> truncate_with_ellipsis('Hello world! ', 6)
        'Hello...' # trailing whitespace is deleted by default (instead of 'Hello ...')
    """
    if len(input_string) <= max_length:
        return input_string
    else:
        input_string = input_string[:max_length]
        if del_blank:
            input_string = input_string.rstrip()
        return input_string + ellipsis


def replace_multiple_substrings(
        input_string: str,
        replacements: Dict[str, str]
) -> str:
    """
    Replaces multiple substrings in a string based on a dictionary of replacements.

    Args:
        input_string (str): The original string to perform replacements on.
        replacements (dict): A dictionary where keys are substrings to be replaced and values are their replacements.

    Returns:
        str: The modified string with all replacements applied.

    Example:
        >>> replacements = {
            'apple': 'orange',
            'banana': 'grape',
            'cherry': 'melon'
        }
        >>>  original_string = 'I have an apple, a banana, and a cherry.'
        >>> replace_multiple_substrings(original_string, replacements)
        'I have an orange, a grape, and a melon.'
    """
    if not replacements:
        return input_string
    pattern = re.compile("|".join([re.escape(k) for k in sorted(replacements, key=len, reverse=True)]), flags=re.DOTALL)
    return pattern.sub(lambda x: replacements[x.group(0)], input_string)


def anonymize_sentence(
        input_string: str,
        # keep_partial_word: bool = False,
        anonymized_char: str = '*',
        # erase_ratio: float = 0.5
) -> str:
    """
    Anonymizes a sentence by replacing all words with a specified character.

    Args:
        input_string (str): The input string to be anonymized.
        anonymized_char (str, optional): The character to use for anonymization. Defaults to '*'.

    Returns:
        str: The anonymized string.

    Examples:
        >>> anonymize_sentence('Hello World')
        '***** *****'
        >>> anonymize_sentence('Alice is 28 years old.')
        '***** ** ** ***** ***.'
        >>> anonymize_sentence('This is a sample sentence with some 123 numbers and special characters!')
        '**** ** * ****** ******** **** **** *** ******* *** ******* **********!'
    """
    words = re.findall(r'\b\w+\b|\W', input_string)
    result = []

    for word in words:
        if word.isalnum():
            replacement = anonymized_char * len(word)
            word = replacement
        result.append(word)

    return ''.join(result)


def text_to_html(text: str, replacements: [str] = None) -> str:
    """
    Converts a text string to HTML.

    Args:
        text:
        replacements:

    Returns:
        str: The converted HTML string.

    Examples:
        >>> text_to_html('Hello world!')
        '<p>Hello world!</p>'
        >>> text_to_html('Hello\nworld!')
        '<p>Hello</p><p>world!</p>'
    """
    if not text:
        return ''
    if replacements is None or not replacements:
        replacements = ['\n']

    for replacement in replacements:
        text = text.replace(replacement, '</p><p>')
    text = f'<p>{text}</p>'

    return text
